@@ -77,19 +77,20 @@ impl State {
         let state = &mut ctx.editor.mode.normal_state;
         state.is_recording_auto_macro = false;
         match keys.next(&ctx.editor.buffered_keys) {
-            Key::Char('q') => match ctx.editor.recording_macro.take() {
-                Some(_) => ctx.editor.recording_macro = None,
-                None => match keys.next(&ctx.editor.buffered_keys) {
-                    Key::None => return Some(ModeOperation::Pending),
-                    Key::Char(c) => {
-                        if let Some(key) = RegisterKey::from_char(c) {
-                            ctx.editor.registers.get_mut(key).clear();
-                            ctx.editor.recording_macro = Some(key);
+            Key::Char('q') => {
+                if ctx.editor.recording_macro.take().is_none() {
+                    match keys.next(&ctx.editor.buffered_keys) {
+                        Key::None => return Some(ModeOperation::Pending),
+                        Key::Char(c) => {
+                            if let Some(key) = RegisterKey::from_char(c) {
+                                ctx.editor.registers.get_mut(key).clear();
+                                ctx.editor.recording_macro = Some(key);
+                            }
                         }
+                        _ => (),
                     }
-                    _ => (),
-                },
-            },
+                }
+            }
             Key::Char('Q') => {
                 ctx.editor.recording_macro = None;
                 match keys.next(&ctx.editor.buffered_keys) {
@@ -446,10 +447,11 @@ impl State {
                             );
                             let buffer_view = ctx.editor.buffer_views.get_mut(handle)?;
                             let buffer = ctx.editor.buffers.get(buffer_view.buffer_handle)?;
+                            let buffer = buffer.content();
                             let line_index = state.count - 1;
                             let mut position = BufferPosition::line_col(line_index as _, 0);
-                            let (first_word, _, mut right_words) =
-                                buffer.content().words_from(position);
+                            position = buffer.saturate_position(position);
+                            let (first_word, _, mut right_words) = buffer.words_from(position);
                             if first_word.kind == WordKind::Whitespace {
                                 if let Some(word) = right_words.next() {
                                     position = word.position;
@@ -1089,11 +1091,11 @@ impl State {
                     let cursor_count = cursors[..].len();
                     let offset = state.count.max(1) as usize;
                     cursors.set_main_cursor_index((index + offset) % cursor_count);
-                    ctx.editor
-                        .mode
-                        .normal_state
-                        .last_copy_ranges
-                        .rotate_right(offset);
+                    let ranges = &mut ctx.editor.mode.normal_state.last_copy_ranges;
+                    if !ranges.is_empty() {
+                        let offset = offset % ranges.len();
+                        ranges.rotate_right(offset);
+                    }
                 }
                 Key::Char('p') => {
                     let cursors = &mut ctx.editor.buffer_views.get_mut(handle)?.cursors;
@@ -1102,11 +1104,11 @@ impl State {
                     let cursor_count = cursors[..].len();
                     let offset = state.count.max(1) as usize % cursor_count;
                     cursors.set_main_cursor_index((index + cursor_count - offset) % cursor_count);
-                    ctx.editor
-                        .mode
-                        .normal_state
-                        .last_copy_ranges
-                        .rotate_left(offset);
+                    let ranges = &mut ctx.editor.mode.normal_state.last_copy_ranges;
+                    if !ranges.is_empty() {
+                        let offset = offset % ranges.len();
+                        ranges.rotate_left(offset);
+                    }
                 }
                 Key::Char('f') => read_line::filter_cursors::enter_filter_mode(ctx),
                 Key::Char('F') => read_line::filter_cursors::enter_except_mode(ctx),
@@ -1126,100 +1128,44 @@ impl State {
             },
             Key::Char('s') => read_line::search::enter_mode(ctx),
             Key::Char('y') => {
-                let state = &mut ctx.editor.mode.normal_state;
-                let buffer_view = ctx.editor.buffer_views.get(handle)?;
-                let mut buf = ctx.editor.string_pool.acquire();
-                buffer_view.append_selection_text(
-                    &ctx.editor.buffers,
-                    &mut buf,
-                    &mut state.last_copy_ranges,
-                );
-                if !buf.is_empty() {
-                    ctx.platform.write_to_clipboard(&buf);
-                    state.last_copy_hash = hash_bytes(buf.bytes());
+                let mut text = ctx.editor.string_pool.acquire();
+                copy_text(ctx, handle, &mut text);
+                if !text.is_empty() {
+                    ctx.platform.write_to_clipboard(&text);
                 }
-                ctx.editor.string_pool.release(buf);
-                state.movement_kind = CursorMovementKind::PositionAndAnchor;
+                ctx.editor.string_pool.release(text);
             }
             Key::Char('Y') => {
-                let buffer_view = ctx.editor.buffer_views.get(handle)?;
-                buffer_view.delete_text_in_cursor_ranges(
-                    &mut ctx.editor.buffers,
-                    &mut ctx.editor.word_database,
-                    &mut ctx.editor.events,
-                );
-
-                state.movement_kind = CursorMovementKind::PositionAndAnchor;
-                state.is_recording_auto_macro = false;
-
-                ctx.editor.trigger_event_handlers(ctx.platform, ctx.clients);
-
-                let mut buf = ctx.editor.string_pool.acquire();
-                let buffer_view = ctx.editor.buffer_views.get(handle)?;
-                ctx.platform.read_from_clipboard(&mut buf);
-                let hash = ctx.editor.mode.normal_state.last_copy_hash;
-                let ranges = &ctx.editor.mode.normal_state.last_copy_ranges[..];
-                let cursors = &buffer_view.cursors[..];
-                if hash == hash_bytes(buf.bytes()) && ranges.len() == cursors.len() {
-                    if let Some(buffer) = ctx.editor.buffers.get_mut(buffer_view.buffer_handle) {
-                        for (range, cursor) in ranges.iter().zip(cursors.iter()).rev() {
-                            let text = &buf[range.0 as usize..range.1 as usize];
-                            buffer.insert_text(
-                                &mut ctx.editor.word_database,
-                                cursor.position,
-                                text,
-                                &mut ctx.editor.events,
-                            );
-                        }
-                    }
-                } else {
-                    buffer_view.insert_text_at_cursor_positions(
-                        &mut ctx.editor.buffers,
-                        &mut ctx.editor.word_database,
-                        &buf,
-                        &mut ctx.editor.events,
-                    );
-                }
-                ctx.editor.string_pool.release(buf);
-
-                let buffer_view = ctx.editor.buffer_views.get(handle)?;
-                ctx.editor
-                    .buffers
-                    .get_mut(buffer_view.buffer_handle)?
-                    .commit_edits();
+                let mut text = ctx.editor.string_pool.acquire();
+                ctx.platform.read_from_clipboard(&mut text);
+                paste_text(ctx, handle, &text);
+                ctx.editor.string_pool.release(text);
                 return None;
             }
             Key::Ctrl('y') => match keys.next(&ctx.editor.buffered_keys) {
                 Key::None => return Some(ModeOperation::Pending),
                 Key::Char(c) => {
-                    let buffer_view = ctx.editor.buffer_views.get(handle)?;
-                    buffer_view.delete_text_in_cursor_ranges(
-                        &mut ctx.editor.buffers,
-                        &mut ctx.editor.word_database,
-                        &mut ctx.editor.events,
-                    );
-
-                    state.movement_kind = CursorMovementKind::PositionAndAnchor;
-                    state.is_recording_auto_macro = false;
-
-                    ctx.editor.trigger_event_handlers(ctx.platform, ctx.clients);
-
-                    let buffer_view = ctx.editor.buffer_views.get(handle)?;
-                    if let Some(key) = RegisterKey::from_char(c) {
-                        let register = ctx.editor.registers.get(key);
-                        buffer_view.insert_text_at_cursor_positions(
-                            &mut ctx.editor.buffers,
-                            &mut ctx.editor.word_database,
-                            register,
-                            &mut ctx.editor.events,
-                        );
+                    let key = c.to_ascii_lowercase();
+                    if key == c {
+                        if let Some(key) = RegisterKey::from_char(key) {
+                            let mut text = ctx.editor.string_pool.acquire();
+                            copy_text(ctx, handle, &mut text);
+                            if !text.is_empty() {
+                                let register = ctx.editor.registers.get_mut(key);
+                                register.clear();
+                                register.push_str(&text);
+                            }
+                            ctx.editor.string_pool.release(text);
+                        }
+                    } else {
+                        if let Some(key) = RegisterKey::from_char(key) {
+                            let register = ctx.editor.registers.get(key);
+                            let text = ctx.editor.string_pool.acquire_with(register);
+                            paste_text(ctx, handle, &text);
+                            ctx.editor.string_pool.release(text);
+                            return None;
+                        }
                     }
-                    let buffer_view = ctx.editor.buffer_views.get(handle)?;
-                    ctx.editor
-                        .buffers
-                        .get_mut(buffer_view.buffer_handle)?
-                        .commit_edits();
-                    return None;
                 }
                 _ => (),
             },
@@ -1331,6 +1277,72 @@ impl ModeState for State {
     }
 }
 
+fn copy_text(
+    ctx: &mut ModeContext,
+    buffer_view_handle: BufferViewHandle,
+    text: &mut String,
+) -> Option<()> {
+    let state = &mut ctx.editor.mode.normal_state;
+    let buffer_view = ctx.editor.buffer_views.get(buffer_view_handle)?;
+    buffer_view.append_selection_text(&ctx.editor.buffers, text, &mut state.last_copy_ranges);
+    if !text.is_empty() {
+        state.last_copy_hash = hash_bytes(text.bytes());
+    }
+    state.movement_kind = CursorMovementKind::PositionAndAnchor;
+    None
+}
+
+fn paste_text(
+    ctx: &mut ModeContext,
+    buffer_view_handle: BufferViewHandle,
+    text: &str,
+) -> Option<()> {
+    let state = &mut ctx.editor.mode.normal_state;
+    let buffer_view = ctx.editor.buffer_views.get(buffer_view_handle)?;
+    buffer_view.delete_text_in_cursor_ranges(
+        &mut ctx.editor.buffers,
+        &mut ctx.editor.word_database,
+        &mut ctx.editor.events,
+    );
+
+    state.movement_kind = CursorMovementKind::PositionAndAnchor;
+    state.is_recording_auto_macro = false;
+
+    ctx.editor.trigger_event_handlers(ctx.platform, ctx.clients);
+
+    let buffer_view = ctx.editor.buffer_views.get(buffer_view_handle)?;
+    let hash = ctx.editor.mode.normal_state.last_copy_hash;
+    let ranges = &ctx.editor.mode.normal_state.last_copy_ranges[..];
+    let cursors = &buffer_view.cursors[..];
+    if hash == hash_bytes(text.bytes()) && ranges.len() == cursors.len() {
+        if let Some(buffer) = ctx.editor.buffers.get_mut(buffer_view.buffer_handle) {
+            for (range, cursor) in ranges.iter().zip(cursors.iter()).rev() {
+                let text = &text[range.0 as usize..range.1 as usize];
+                buffer.insert_text(
+                    &mut ctx.editor.word_database,
+                    cursor.position,
+                    text,
+                    &mut ctx.editor.events,
+                );
+            }
+        }
+    } else {
+        buffer_view.insert_text_at_cursor_positions(
+            &mut ctx.editor.buffers,
+            &mut ctx.editor.word_database,
+            &text,
+            &mut ctx.editor.events,
+        );
+    }
+
+    let buffer_view = ctx.editor.buffer_views.get(buffer_view_handle)?;
+    ctx.editor
+        .buffers
+        .get_mut(buffer_view.buffer_handle)?
+        .commit_edits();
+    None
+}
+
 fn find_char(ctx: &mut ModeContext, forward: bool) -> Option<()> {
     let state = &ctx.editor.mode.normal_state;
     let skip;
